<!doctype html>
<html lang="en">

<head>
    <title>405 Method Not Allowed</title>
</head>

<body>
<p>Should of GET when you POST, or POST when you GET</p>
<p>location of this file: app/structures/default_theme/errors/405.html</p>
<p>Being rendered from: app/flask/routes/errors.py</p>
</body>
</html>
